use crate::synchronizations;
use once_cell::sync::OnceCell;
use std::sync::Arc;
use tokio_cron_scheduler::{Job, JobScheduler};

static G_INSTANCE: OnceCell<Arc<Scheduler>> = OnceCell::new();

///
pub struct Scheduler {
  sched: JobScheduler,
  vendor_synchronizer: synchronizations::vendor::synchronizer::Synchronizer,
  game_synchronizer: synchronizations::game::synchronizer::Synchronizer,
}

impl std::fmt::Debug for Scheduler {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Scheduler of api.kgon.identity").finish()
  }
}

impl Scheduler {
  pub fn get_instance(
    sched: JobScheduler,
    vendor_synchronizer: synchronizations::vendor::synchronizer::Synchronizer,
    game_synchronizer: synchronizations::game::synchronizer::Synchronizer,
  ) -> Result<&'static Arc<Scheduler>, Box<dyn std::error::Error>> {
    let instance = G_INSTANCE
      .get_or_try_init(|| -> Result<Arc<Scheduler>, Box<dyn std::error::Error>> {
        let s = Scheduler {
          sched,
          vendor_synchronizer,
          game_synchronizer,
        };

        Ok(Arc::new(s))
      })
      .expect("");

    Ok(instance)
  }

  pub async fn queue(&'static self) -> Result<(), std::boxed::Box<dyn std::error::Error>> {
    self.vendors_and_games().await?;

    Ok(())
  }

  async fn vendors_and_games(&'static self) -> Result<(), Box<dyn std::error::Error>> {
    let j_synchronization = Job::new_async("0 0/3 * * * ?", move |_uuid, _l| {
      Box::pin(async move {
        self
          .vendor_synchronizer
          .vendors()
          .await
          .expect("vendor_synchronizer.vendors");

        self
          .game_synchronizer
          .games()
          .await
          .expect("game_synchronizer.games");
      })
    })?;

    self.sched.add(j_synchronization).await?;

    Ok(())
  }
}
